# Trackers definition
# Copyright (C) 2010  Sylvain Beucler
#
# This file is part of Savane.
# 
# Savane is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
# 
# Savane is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# 
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
The Field definition cannot be changed, so it's been made static
instead of stored in the database.

Here's the previous model definition.  See also FieldUsage, which is
still used for non-default (i.e. override) values.

    class Meta:
        unique_together = (('tracker', 'name'),)
        verbose_name = _("field")
        verbose_name_plural = _("fields")

    DISPLAY_TYPE_CHOICES = (('DF', _('date field')),
                            ('SB', _('select box')),
                            ('TA', _('text area')),
                            ('TF', _('text field')),)
    SCOPE_CHOICES = (('S', _('system')), # user cannot modify related FieldValue's (TF)
                     ('P', _('project')),)  # user can modify related FieldValue's (TF)

    tracker = models.ForeignKey('Tracker')
    name = models.CharField(max_length=255, db_index=True)
    display_type = models.CharField(max_length=255, choices=DISPLAY_TYPE_CHOICES)
    display_size = models.CharField(max_length=255)
      # DF: unused
      # SB: unused
      # TA: cols/rows
      # TF: visible_length/max_length
    label  = models.CharField(max_length=255)
    description = models.TextField()

    # Field values can be changed (if TF)
    scope = models.CharField(max_length=1, choices=SCOPE_CHOICES)
    # Field cannot be hidden (but can be made optional)
    required = models.BooleanField(help_text=_("field cannot be disabled in configuration"))
    # Default value (fields can always override this except for 'summary' and 'details', cf. 'special')
    empty_ok = models.CharField(max_length=1, choices=EMPTY_OK_CHOICES,
                                default='0')
    # Default value
    keep_history = models.BooleanField()
    # Field cannot be made optional (displayed unless 'bug_id' and 'group_id')
    # Also, field are not displayed (filled by the system) - except for 'summary', 'comment_type' and 'details'
    # (consequently, they cannot be customized in any way, except for 'summary' and 'details' where you can only customize the display size)
    special = models.BooleanField()
    # Field may change label and description
    custom = models.BooleanField(help_text=_("let the user change the label and description"))
"""

from django.utils.translation import ugettext, ugettext_lazy as _
from copy import deepcopy

default_fields = {}

common = {
    'bug_id' : {
        'field_name': 'bug_id',
        'display_type': 'TF',
        'display_size': '6/10',
        'label': _("Item ID"),
        'description': _("Unique item identifier"),
        'scope': 'S',
        'required': 1,
        'empty_ok': 0,
        'keep_history': 0,
        'special': 1,
        'custom': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 10,
    },
    'group_id' : {
        'field_name': 'group_id',
        'display_type': 'TF',
        'display_size': '',
        'label': _("Group ID"),
        'description': _("Unique project identifier"),
        'scope': 'S',
        'required': 1,
        'empty_ok': 0,
        'keep_history': 0,
        'special': 1,
        'custom': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 1,
        'show_on_add_members': 1,
        'rank': 30,
    },
    'submitted_by' : {
        'field_name': 'submitted_by',
        'display_type': 'SB',
        'label': _("Submitted by"),
        'description': _("User who originally submitted the item"),
        'scope': 'S',
        'required': 1,
        'empty_ok': 1,
        'keep_history': 0,
        'special': 1,
        'custom': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 20,
        'transition_default_auth': 'A',
    },
    'date' : {
        'field_name': 'date',
        'display_type': 'DF',
        'label': _("Submitted on"),
        'description': _("Date and time of the initial submission"),
        'scope': 'S',
        'required': 1,
        'empty_ok': 0,
        'keep_history': 0,
        'special': 1,
        'custom': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 40,
    },
    'close_date' : {
        'field_name': 'close_date',
        'display_type': 'DF',
        'label': _("Closed on"),
        'description': _("Date and time when the item  status was changed to 'Closed'"),
        'scope': 'S',
        'required': 1,
        'empty_ok': 1,
        'keep_history': 0,
        'special': 1,
        'custom': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 50,
    },
    'status_id' : {
        'field_name': 'status_id',
        'display_type': 'SB',
        'label': _("Open/Closed"),
        'description': _("Most basic status"),
        'scope': 'S',
        'required': 1,
        'empty_ok': 0,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 600,
        'transition_default_auth': 'A',
    },
    'severity' : {
        'field_name': 'severity',
        'display_type': 'SB',
        'label': _("Severity"),
        'description': _("Impact of the item on the system (Critical, Major,...)"),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 1,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 1,
        'show_on_add_members': 1,
        'rank': 200,
        'transition_default_auth': 'A',
    },
    'category_id' : {
        'field_name': 'category_id',
        'display_type': 'SB',
        'label': _("Category"),
        'description': _("Generally high level modules or functionalities of the software (e.g. User interface, Configuration Manager, etc)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 1,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 1,
        'show_on_add_members': 1,
        'rank': 100,
        'transition_default_auth': 'A',
    },
    'assigned_to' : {
        'field_name': 'assigned_to',
        'display_type': 'SB',
        'label': _("Assigned to"),
        'description': _("Who is in charge of handling this item"),
        'scope': 'S',
        'required': 1,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 1,
        'rank': 500,
        'transition_default_auth': 'A',
    },
    'summary' : {
        'field_name': 'summary',
        'display_type': 'TF',
        'display_size': '65/120',
        'label': _("Summary"),
        'description': _("One line description of the item"),
        'scope': 'S',
        'required': 1,
        'empty_ok': 0,
        'keep_history': 1,
        'special': 1,
        'custom': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 1,
        'show_on_add_members': 1,
        'rank': 700000,
    },
    'details' : {
        'field_name': 'details',
        'display_type': 'TA',
        'display_size': '65/20',
        'label': _("Original Submission"),
        'description': _("Full description of the item"),
        'scope': 'S',
        'required': 1,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 1,
        'custom': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 1,
        'show_on_add_members': 1,
        'rank': 700001,
    },
    'bug_group_id' : {
        'field_name': 'bug_group_id',
        'display_type': 'SB',
        'label': _("Item Group"),
        'description': _("Characterizes the nature of the item (e.g. Crash Error, Documentation Typo, Installation Problem, etc"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 1,
        'show_on_add_members': 1,
        'rank': 300,
        'transition_default_auth': 'A',
    },
    'resolution_id' : {
        'field_name': 'resolution_id',
        'display_type': 'SB',
        'label': _("Status"),
        'description': _("Current resolution of the item"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 1,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 1,
        'rank': 400,
        'transition_default_auth': 'A',
    },
    'privacy' : {
        'field_name': 'privacy',
        'display_type': 'SB',
        'label': _("Privacy"),
        'description': _("Determines whether the item can be seen by members of the project only or anybody."),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 1,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 1,
        'show_on_add_members': 1,
        'rank': 402,
        'transition_default_auth': 'A',
    },
    'vote' : {
        'field_name': 'vote',
        'display_type': 'TF',
        'display_size': '6/10',
        'label': _("Votes"),
        'description': _("How many votes this item received."),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 0,
        'special': 1,
        'custom': 0,
        'use_it': 1,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 405,
    },
    'category_version_id' : {
        'field_name': 'category_version_id',
        'display_type': 'SB',
        'label': _("Component Version"),
        'description': _("Version of the System Component (aka Item Category) impacted by the item"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 1000,
        'transition_default_auth': 'A',
    },
    'platform_version_id' : {
        'field_name': 'platform_version_id',
        'display_type': 'SB',
        'label': _("Operating System"),
        'description': _("Operating System impacted by the issue"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 1100,
        'transition_default_auth': 'A',
    },
    'reproducibility_id' : {
        'field_name': 'reproducibility_id',
        'display_type': 'SB',
        'label': _("Reproducibility"),
        'description': _("How easy it is to reproduce the item"),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 1200,
        'transition_default_auth': 'A',
    },
    'size_id' : {
        'field_name': 'size_id',
        'display_type': 'SB',
        'label': _("Size (loc)"),
        'description': _("Estimated size of the code to be developed or reworked to fix the item"),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 1300,
        'transition_default_auth': 'A',
    },
    'fix_release_id' : {
        'field_name': 'fix_release_id',
        'display_type': 'SB',
        'label': _("Fixed Release"),
        'description': _("Release in which the item was actually fixed"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 1400,
        'transition_default_auth': 'A',
    },
    'comment_type_id' : {
        'field_name': 'comment_type_id',
        'display_type': 'SB',
        'label': _("Comment Type"),
        'description': _("Specify the nature of the follow up comment attached to this item"),
        'scope': 'P',
        'required': 1,
        'empty_ok': 1,
        'keep_history': 0,
        'special': 1,
        'custom': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 1500,
        'transition_default_auth': 'A',
    },
    'hours' : {
        'field_name': 'hours',
        'display_type': 'TF',
        'display_size': '5/5',
        'label': _("Effort"),
        'description': _("Number of hours of work needed to fix the item"),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 1700,
    },
    'plan_release_id' : {
        'field_name': 'plan_release_id',
        'display_type': 'SB',
        'label': _("Planned Release"),
        'description': _("Release in which it is planned to have the item fixed"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 1600,
        'transition_default_auth': 'A',
    },
    'component_version' : {
        'field_name': 'component_version',
        'display_type': 'TF',
        'display_size': '10/40',
        'label': _("Component Version"),
        'description': _("Version of the system component (or work product) impacted by the item (<u>in free text</u>)"),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 1800,
    },
    'fix_release' : {
        'field_name': 'fix_release',
        'display_type': 'TF',
        'display_size': '10/40',
        'label': _("Fixed Release"),
        'description': _("Release in which the bug was actually fixed (<u>in free text</u>)"),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 1900,
    },
    'plan_release' : {
        'field_name': 'plan_release',
        'display_type': 'TF',
        'display_size': '10/40',
        'label': _("Planned Release"),
        'description': _("Release in which it is planned to have the item fixed (<u>in free text</u>)"),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 2000,
    },
    'priority' : {
        'field_name': 'priority',
        'display_type': 'SB',
        'label': _("Priority"),
        'description': _("How quickly the item should be handled"),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 1,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 1,
        'rank': 200,
        'transition_default_auth': 'A',
    },
    'keywords' : {
        'field_name': 'keywords',
        'display_type': 'TF',
        'display_size': '60/120',
        'label': _("Keywords"),
        'description': _("List of comma separated keywords associated with this item"),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 3000,
    },
    'release_id' : {
        'field_name': 'release_id',
        'display_type': 'SB',
        'label': _("Release"),
        'description': _("Release (global version number) impacted by the item"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 800,
        'transition_default_auth': 'A',
    },
    'release' : {
        'field_name': 'release',
        'display_type': 'TF',
        'display_size': '10/40',
        'label': _("Release"),
        'description': _("Release (global version number) impacted by the item (<u>in free text</u>)"),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 800,
    },
    'originator_name' : {
        'field_name': 'originator_name',
        'display_type': 'TF',
        'display_size': '20/40',
        'label': _("Originator Name"),
        'description': _("Name of the person who submitted the item (if different from the submitter field)"),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 550,
    },
    'originator_email' : {
        'field_name': 'originator_email',
        'display_type': 'TF',
        'display_size': '20/40',
        'label': _("Originator Email"),
        'description': _("Email address of the person who submitted the item (if different from the submitter field, add address to CC list)"),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 1,
        'show_on_add_anonymous': 1,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 560,
    },
    'originator_phone' : {
        'field_name': 'originator_phone',
        'display_type': 'TF',
        'display_size': '10/40',
        'label': _("Originator Phone"),
        'description': _("Phone number of the person who submitted the item"),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 570,
    },
    'percent_complete' : {
        'field_name': 'percent_complete',
        'display_type': 'SB',
        'label': _("Percent Complete"),
        'description': _(""),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 500,
        'transition_default_auth': 'A',
    },
    'custom_tf1' : {
        'field_name': 'custom_tf1',
        'display_type': 'TF',
        'display_size': '10/15',
        'label': _("Custom Text Field #1"),
        'description': _("Customizable Text Field (one line, up to 255 characters"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 30000,
    },
    'custom_tf2' : {
        'field_name': 'custom_tf2',
        'display_type': 'TF',
        'display_size': '10/15',
        'label': _("Custom Text Field #2"),
        'description': _("Customizable Text Field (one line, up to 255 characters"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 30100,
    },
    'custom_tf3' : {
        'field_name': 'custom_tf3',
        'display_type': 'TF',
        'display_size': '10/15',
        'label': _("Custom Text Field #3"),
        'description': _("Customizable Text Field (one line, up to 255 characters"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 30200,
    },
    'custom_tf4' : {
        'field_name': 'custom_tf4',
        'display_type': 'TF',
        'display_size': '10/15',
        'label': _("Custom Text Field #4"),
        'description': _("Customizable Text Field (one line, up to 255 characters"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 30300,
    },
    'custom_tf5' : {
        'field_name': 'custom_tf5',
        'display_type': 'TF',
        'display_size': '10/15',
        'label': _("Custom Text Field #5"),
        'description': _("Customizable Text Field (one line, up to 255 characters"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 30400,
    },
    'custom_tf6' : {
        'field_name': 'custom_tf6',
        'display_type': 'TF',
        'display_size': '10/15',
        'label': _("Custom Text Field #6"),
        'description': _("Customizable Text Field (one line, up to 255 characters"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 30500,
    },
    'custom_tf7' : {
        'field_name': 'custom_tf7',
        'display_type': 'TF',
        'display_size': '10/15',
        'label': _("Custom Text Field #7"),
        'description': _("Customizable Text Field (one line, up to 255 characters"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 30600,
    },
    'custom_tf8' : {
        'field_name': 'custom_tf8',
        'display_type': 'TF',
        'display_size': '10/15',
        'label': _("Custom Text Field #8"),
        'description': _("Customizable Text Field (one line, up to 255 characters"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 30700,
    },
    'custom_tf9' : {
        'field_name': 'custom_tf9',
        'display_type': 'TF',
        'display_size': '10/15',
        'label': _("Custom Text Field #9"),
        'description': _("Customizable Text Field (one line, up to 255 characters"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 30800,
    },
    'custom_tf10' : {
        'field_name': 'custom_tf10',
        'display_type': 'TF',
        'display_size': '10/15',
        'label': _("Custom Text Field #10"),
        'description': _("Customizable Text Field (one line, up to 255 characters"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 30900,
    },
    'custom_ta1' : {
        'field_name': 'custom_ta1',
        'display_type': 'TA',
        'display_size': '60/3',
        'label': _("Custom Text Area #1"),
        'description': _("Customizable Text Area (multi-line text)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 40000,
    },
    'custom_ta2' : {
        'field_name': 'custom_ta2',
        'display_type': 'TA',
        'display_size': '60/3',
        'label': _("Custom Text Area #2"),
        'description': _("Customizable Text Area (multi-line text)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 40100,
    },
    'custom_ta3' : {
        'field_name': 'custom_ta3',
        'display_type': 'TA',
        'display_size': '60/3',
        'label': _("Custom Text Area #3"),
        'description': _("Customizable Text Area (multi-line text)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 40200,
    },
    'custom_ta4' : {
        'field_name': 'custom_ta4',
        'display_type': 'TA',
        'display_size': '60/3',
        'label': _("Custom Text Area #4"),
        'description': _("Customizable Text Area (multi-line text)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 40300,
    },
    'custom_ta5' : {
        'field_name': 'custom_ta5',
        'display_type': 'TA',
        'display_size': '60/3',
        'label': _("Custom Text Area #5"),
        'description': _("Customizable Text Area (multi-line text)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 40400,
    },
    'custom_ta6' : {
        'field_name': 'custom_ta6',
        'display_type': 'TA',
        'display_size': '60/3',
        'label': _("Custom Text Area #6"),
        'description': _("Customizable Text Area (multi-line text)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 40500,
    },
    'custom_ta7' : {
        'field_name': 'custom_ta7',
        'display_type': 'TA',
        'display_size': '60/3',
        'label': _("Custom Text Area #7"),
        'description': _("Customizable Text Area (multi-line text)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 40600,
    },
    'custom_ta8' : {
        'field_name': 'custom_ta8',
        'display_type': 'TA',
        'display_size': '60/3',
        'label': _("Custom Text Area #8"),
        'description': _("Customizable Text Area (multi-line text)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 40700,
    },
    'custom_ta9' : {
        'field_name': 'custom_ta9',
        'display_type': 'TA',
        'display_size': '60/3',
        'label': _("Custom Text Area #9"),
        'description': _("Customizable Text Area (multi-line text)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 40800,
    },
    'custom_ta10' : {
        'field_name': 'custom_ta10',
        'display_type': 'TA',
        'display_size': '60/3',
        'label': _("Custom Text Area #10"),
        'description': _("Customizable Text Area (multi-line text)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 40900,
    },
    'custom_sb1' : {
        'field_name': 'custom_sb1',
        'display_type': 'SB',
        'label': _("Custom Select Box #1"),
        'description': _("Customizable Select Box (pull down menu with predefined values)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 50000,
        'transition_default_auth': 'A',
    },
    'custom_sb2' : {
        'field_name': 'custom_sb2',
        'display_type': 'SB',
        'label': _("Custom Select Box #2"),
        'description': _("Customizable Select Box (pull down menu with predefined values)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 50100,
        'transition_default_auth': 'A',
    },
    'custom_sb3' : {
        'field_name': 'custom_sb3',
        'display_type': 'SB',
        'label': _("Custom Select Box #3"),
        'description': _("Customizable Select Box (pull down menu with predefined values)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 50200,
        'transition_default_auth': 'A',
    },
    'custom_sb4' : {
        'field_name': 'custom_sb4',
        'display_type': 'SB',
        'label': _("Custom Select Box #4"),
        'description': _("Customizable Select Box (pull down menu with predefined values)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 50300,
        'transition_default_auth': 'A',
    },
    'custom_sb5' : {
        'field_name': 'custom_sb5',
        'display_type': 'SB',
        'label': _("Custom Select Box #5"),
        'description': _("Customizable Select Box (pull down menu with predefined values)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 50400,
        'transition_default_auth': 'A',
    },
    'custom_sb6' : {
        'field_name': 'custom_sb6',
        'display_type': 'SB',
        'label': _("Custom Select Box #6"),
        'description': _("Customizable Select Box (pull down menu with predefined values)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 50500,
        'transition_default_auth': 'A',
    },
    'custom_sb7' : {
        'field_name': 'custom_sb7',
        'display_type': 'SB',
        'label': _("Custom Select Box #7"),
        'description': _("Customizable Select Box (pull down menu with predefined values)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 50600,
        'transition_default_auth': 'A',
    },
    'custom_sb8' : {
        'field_name': 'custom_sb8',
        'display_type': 'SB',
        'label': _("Custom Select Box #8"),
        'description': _("Customizable Select Box (pull down menu with predefined values)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 50700,
        'transition_default_auth': 'A',
    },
    'custom_sb9' : {
        'field_name': 'custom_sb9',
        'display_type': 'SB',
        'label': _("Custom Select Box #9"),
        'description': _("Customizable Select Box (pull down menu with predefined values)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 50800,
        'transition_default_auth': 'A',
    },
    'custom_sb10' : {
        'field_name': 'custom_sb10',
        'display_type': 'SB',
        'label': _("Custom Select Box #10"),
        'description': _("Customizable Select Box (pull down menu with predefined values)"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 50900,
        'transition_default_auth': 'A',
    },
    'custom_df1' : {
        'field_name': 'custom_df1',
        'display_type': 'DF',
        'label': _("Custom Date Field #1"),
        'description': _("Customizable Date Field"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 60000,
    },
    'custom_df2' : {
        'field_name': 'custom_df2',
        'display_type': 'DF',
        'label': _("Custom Date Field #2"),
        'description': _("Customizable Date Field"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 60100,
    },
    'custom_df3' : {
        'field_name': 'custom_df3',
        'display_type': 'DF',
        'label': _("Custom Date Field #3"),
        'description': _("Customizable Date Field"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 60200,
    },
    'custom_df4' : {
        'field_name': 'custom_df4',
        'display_type': 'DF',
        'label': _("Custom Date Field #4"),
        'description': _("Customizable Date Field"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 60300,
    },
    'custom_df5' : {
        'field_name': 'custom_df5',
        'display_type': 'DF',
        'label': _("Custom Date Field #5"),
        'description': _("Customizable Date Field"),
        'scope': 'P',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 1,
        'use_it': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 60400,
    },
    'discussion_lock' : {
        'field_name': 'discussion_lock',
        'display_type': 'SB',
        'label': _("Discussion Lock"),
        'description': _("Determines whether comments can still be added to the item"),
        'scope': 'S',
        'required': 1,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 0,
        'show_on_add_members': 0,
        'rank': 800,
        'transition_default_auth': 'A',
    },
    'planned_close_date' : {
        'field_name': 'planned_close_date',
        'display_type': 'DF',
        'label': _("Should be Finished on"),
        'description': _("Date and time when the item should be completed"),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 1,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 1,
        'show_on_add_members': 1,
        'rank': 56,
    },
    'planned_starting_date' : {
        'field_name': 'planned_starting_date',
        'display_type': 'DF',
        'label': _("Should Start On"),
        'description': _("Date and time when someone should start working on the item"),
        'scope': 'S',
        'required': 0,
        'empty_ok': 1,
        'keep_history': 1,
        'special': 0,
        'custom': 0,
        'use_it': 1,
        'show_on_add_anonymous': 0,
        'show_on_add_connected': 1,
        'show_on_add_members': 1,
        'rank': 55,
    },
}

default_fields['bugs']    = deepcopy(common)
default_fields['patch']   = deepcopy(common)
default_fields['support'] = deepcopy(common)
default_fields['task']    = deepcopy(common)

# A couple changes per-tracker in the default configuration
#submitted_by     92 -> (use_it=0)(patch,support) | (use_it=1)(bugs,task)
default_fields['patch']['submitted_by']['use_it'] = default_fields['support']['submitted_by']['use_it'] = 0
#severity    102 -> (use_it,show_on_add,show_on_add_members=0)(patch,task) | (use_it,show_on_add,show_on_add_members=1)(bugs,support)
default_fields['patch']['severity']['use_it'] = default_fields['task']['severity']['use_it'] = 0
default_fields['patch']['severity']['show_on_add'] = default_fields['task']['severity']['show_on_add'] = 0
default_fields['patch']['severity']['show_on_add_members'] = default_fields['task']['severity']['show_on_add_members'] = 0
#bug_group_id    107 -> (show_on_add,show_on_add_members=0)(patch,support) | (show_on_add,show_on_add_members=1)(bugs,task) [but use_it=0]
#default_fields['patch']['bug_group_id']['show_on_add'] = default_fields['support']['bug_group_id']['show_on_add'] = 0
#default_fields['patch']['bug_group_id']['show_on_add_members'] = default_fields['support']['bug_group_id']['show_on_add_members'] = 0
#platform_version_id    201 -> (use_it,show_on_add,show_on_add_members=0)(bugs,patch,task) | (use_it,show_on_add,show_on_add_members=1)(support)
default_fields['support']['platform_version_id']['use_it'] = 1
default_fields['support']['platform_version_id']['show_on_add'] = 1
default_fields['support']['platform_version_id']['show_on_add_members'] = 1
#hours    201 -> (use_it,show_on_add,show_on_add_members=0)(bugs,patch,support) | (use_it,show_on_add,show_on_add_members=1)(task)
default_fields['task']['hours']['use_it'] = 1
default_fields['task']['hours']['show_on_add'] = 1
default_fields['task']['hours']['show_on_add_members'] = 1
#priority    211 -> (show_on_add,show_on_add_members,place)
#           bugs: 0,1,200
#           patch: 1,1,150
default_fields['patch']['priority']['show_on_add'] = 1
default_fields['patch']['priority']['show_on_add_members'] = 1
default_fields['patch']['priority']['place'] = 150
#           support: 0,0,150
default_fields['support']['priority']['show_on_add'] = 0
default_fields['support']['priority']['show_on_add_members'] = 0
default_fields['support']['priority']['place'] = 150
#           task: 1,1,200
default_fields['task']['priority']['show_on_add'] = 1
default_fields['task']['priority']['show_on_add_members'] = 1
default_fields['task']['priority']['place'] = 200
#originator_email    216 -> (use_it,show_on_add=0)(task) | (use_it=1,show_on_add=2)(bugs,patch,support)
default_fields['task']['originator_email']['use_it'] = 0
default_fields['task']['originator_email']['show_on_add'] = 0
#percent_complete    220 -> (use_it,show_on_add_members=0)(bugs,patch,support) | (use_it,show_on_add_members=1)(task)
default_fields['task']['originator_email']['use_it'] = 1
default_fields['task']['originator_email']['show_on_add_members'] = 1

# 'planned_starting_date' was initially specific to 'task'
default_fields['bugs']   ['planned_starting_date']['use_it'] = 0
default_fields['patch']  ['planned_starting_date']['use_it'] = 0
default_fields['support']['planned_starting_date']['use_it'] = 0
# 'same for planned_close_date'
default_fields['bugs']   ['planned_close_date']['use_it'] = 0
default_fields['patch']  ['planned_close_date']['use_it'] = 0
default_fields['support']['planned_close_date']['use_it'] = 0
